;;; -*- Mode: Lisp; Package: elp-hacks -*-
;;;
;;; Things to make it easier to communicate with Ergo Lambda Prolog
;;; from Lisp.
;;;
;;; Author: Timothy Freeman.  Last Modified Tue Oct  3 20:00:46 1989
;;;
;;; Sccs Id "@(#)elp-hacks.lisp	1.4 10/4/89
;;;
;;; ******************************************************************* ;;;
;;;         (c) Copyrighted 1989 by Carnegie Mellon University.         ;;;
;;;                        All rights reserved.                         ;;;
;;;         This code was written as part of the Ergo project.          ;;;
;;;  If you want to use this code or any Ergo software, please contact  ;;;
;;;			Frank Pfenning (fp@cs.cmu.edu)			;;;
;;; ******************************************************************* ;;;

(in-package :elp-hacks) (use-package :ergolisp)

(export '(parse-lprolog-from-string))
(defun parse-lprolog-from-string (goal)
  (with-input-from-string (s (concatenate 'string goal "."))
    (elp:tlang-call parse-sterm s)))

(export 'syntactic-to-logical)
(ddefun syntactic-to-logical (_)
  (error "Tim is confused."))

(export 'logical-to-syntactic)
(defun logical-to-syntactic (term)
  (lterm:to-slterm term () () () 0))

(declare-constructor syntactic-to-logical (logical-to-syntactic))

(defun mk-lprolog-constant (op1 op2)
  (term:mk-term #^slterm:con
		(list (term:mk-term
		       op1
		       (list (term:mk-term op2 nil))))))

(defun is-lprolog-constant (term operator)
  (dlet ( ((term:mk-term op args) term) )
    (and (oper:oper-equal op #^slterm:con)
	 (dlet ( ((list (term:mk-term op2 _)) args) )
	   (oper:oper-equal op2 operator)))))

(defun ds-lprolog-constant-1 (term)
  (dlet ( ((term:mk-term _
			 (list (term:mk-term res _))) term) )
    res))
  
(defun ds-lprolog-constant-2 (term)
  (dlet ( ((term:mk-term _
			 (list (term:mk-term _
					     (list (term:mk-term res _)))))
	   term) )
    res))

(declare-constructor mk-lprolog-constant
		     (ds-lprolog-constant-1 ds-lprolog-constant-2))

(export 'mk-int)
(defun mk-int (int)
  "Make an lprolog int."
  (mk-lprolog-constant #^slterm:cnumber (oper:mk-num-op int)))

(defun is-int (int)
  (is-lprolog-constant int #^slterm:cnumber))

(defun ds-int (term)
  (dlet ( ((mk-lprolog-constant _ (oper:mk-num-op result))
	   term) )
    result))

(declare-constructor mk-int (ds-int) is-int)

#+regression
(progn
  (assert (equalp (mk-int 3) (parse-lprolog-from-string "3")))
  (assert (is-int (mk-int 3)))
  (assert (not (is-int (parse-lprolog-from-string "\"A\""))))
  (dlet ( ((mk-int x) (mk-int 3)) )
    (assert (= x 3))))

(export 'mk-str)
(defun mk-str (string)
  "Make an lprolog string."
  (mk-lprolog-constant #^slterm:cstring (oper:mk-str-op string)))

(defun is-str (string)
  (is-lprolog-constant string #^slterm:cstring))

(defun ds-str (term)
  (dlet ( ((mk-lprolog-constant _ (oper:mk-str-op result))
	   term) )
    result))

(declare-constructor mk-str (ds-str) is-str)

#+regression
(progn
  (assert (equalp (mk-str "A") (parse-lprolog-from-string "\"A\"")))
  (assert (equal (ds-str (mk-str "A")) "A"))
  (assert (is-str (mk-str "A"))))

(export 'mk-con)
(defun mk-con (sym)
  "Make an lprolog identifier constant."
  (mk-lprolog-constant #^slterm:cnamed (oper:mk-id-op sym)))

(defun is-con (string)
  (is-lprolog-constant string #^slterm:cnamed))

(defun ds-con (term)
  (dlet ( ((mk-lprolog-constant _ (oper:mk-id-op sym))
	   term) )
    sym))
  
(declare-constructor mk-con (ds-con) is-con)

#+regression
(progn
  (assert (equalp (mk-con 'slterm::HI) (parse-lprolog-from-string "HI")))
  (assert (is-con (mk-con 'slterm::HI)))
  (dlet ( ((mk-con x) (mk-con 'slterm::x)) )
    (assert (equal x 'slterm::x))))

(export 'mk-appl)
(defun mk-appl (t1 t2)
  "Make an lprolog application."
  (term:mk-term #^slterm:appl (list t1 t2)))

(defun is-appl (term)
  (dlet ( ((term:mk-term op _) term) )
    (oper:oper-equal op #^slterm:appl)))

(defun ds-appl1 (term)
  (term:term-arg0 term))

(defun ds-appl2 (term)
  (term:term-arg1 term))

(declare-constructor mk-appl (ds-appl1 ds-appl2) is-appl)

#+regression
(progn
  (assert (equalp (mk-appl (mk-str "A") (mk-str "B"))
		  (parse-lprolog-from-string "\"A\" \"B\"")))
  (assert (is-appl (mk-appl (mk-str "A") (mk-str "B"))))
  (assert (not (is-str (mk-appl (mk-str "A") (mk-str "B")))))
  (dlet ( ((mk-appl (mk-str x) (mk-str y))
	   (mk-appl (mk-str "A") (mk-str "B"))) )
    (assert (and (equal x "A") (equal y "B")))))

(export 'mk-nil)
(defun mk-nil () (mk-con 'slterm::|nil|))

(defun is-nil (thing) (equalp thing (mk-nil)))

(declare-constructor mk-nil () is-nil)

#+regression
(assert (equalp (mk-nil) (parse-lprolog-from-string "nil")))

(defun mk-lprolog-two-arg-fun (sym x y)
  "Make a call to an arbitrary lprolog two-argument function.."
  (mk-appl
   (mk-appl (mk-con sym) x)
   y)) 

(defun is-lprolog-two-arg-fun (term sym)
  (and (is-appl term)
       (dlet ( ((mk-appl arg1 _) term) )
	 (and (is-appl arg1)
	      (dlet ( ((mk-appl arg3 _) arg1) )
		(and (is-con arg3)
		     (dlet ( ((mk-con con) arg3) )
		       (eq con sym))))))))

(export 'mk-cons)
(defun mk-cons (x y)
  "Make an lprolog cons."
  (mk-lprolog-two-arg-fun 'slterm::|cons| x y))

(defun is-cons (term)
  (is-lprolog-two-arg-fun term 'slterm::|cons|))

(defun lprolog-arg1 (term)
  (dlet ( ((mk-appl (mk-appl _ x) _) term) )
    x))

(defun lprolog-arg2 (term)
  (dlet ( ((mk-appl _ y) term) )
    y))
    
(declare-constructor mk-cons (lprolog-arg1 lprolog-arg2) is-cons)

#+regression
(progn
  (assert (equalp (mk-cons (mk-nil) (mk-nil))
		  (parse-lprolog-from-string "(cons nil nil)")))
  (assert (is-cons (mk-cons (mk-nil) (mk-nil))))
  (assert (not (is-cons (mk-str "A"))))
  (dlet ( ((mk-cons x y) (mk-cons (mk-str "A") (mk-str "B"))) )
    (assert (equalp x (mk-str "A"))
	    (equalp y (mk-str "B")))))

(export 'mk-rule)
(defun mk-rule (x y)
  "Make an lprolog rule."
  (mk-lprolog-two-arg-fun 'slterm::|rule| x y))

(defun is-rule (term)
  (is-lprolog-two-arg-fun term 'slterm::|rule|))

(declare-constructor mk-rule (lprolog-arg1 lprolog-arg2) is-rule)

