;;; -*- Mode: Lisp; Package: mellowcard -*-
;;;
;;; Cheap hypertext.
;;;
;;; Author: Timothy Freeman.  Last Modified Wed Aug 30 23:15:30 1989
;;;
;;; Sccs Id "@(#)hyperhack.lisp	1.5 8/31/89
;;;
;;; ******************************************************************* ;;;
;;;         (c) Copyrighted 1989 by Carnegie Mellon University.         ;;;
;;;                        All rights reserved.                         ;;;
;;;         This code was written as part of the Ergo project.          ;;;
;;;  If you want to use this code or any Ergo software, please contact  ;;;
;;;			Frank Pfenning (fp@cs.cmu.edu)			;;;
;;; ******************************************************************* ;;;

(in-package :mellowcard) (use-package :ergolisp)

(use-package :adt-obj)

(export 'interpret-fn)
(export 'hypertext-win)
(def-combined-class hypertext-win
  display:title-highlight-parent-term-window
  display:mouse-message-mixin
  ;; The class name "hyper-guts" is due to a sudden burst of
  ;; creativity, not any lingering dissatisfaction with the world.
  (anonymous-class hyper-guts (interpret-fn)
    :documentation "Allow the user to select pieces and then call
interpret-fn on the piece he selected."
    :initable-instance-vars (interpret-fn)
    :default-methods ((interpret-fn 'eval)
		      (display:message-string ""))
    :called-client-methods (display:set-message-string display:destroy-window)
    :user-methods
    ((display:key-or-button (key downp code state part)
       (declare (ignore state))
       (when (and downp (eq key :button))
	 (when (and (or (= code display:*left-button*)
			(= code display:*middle-button*))
		    part)
	   (dlet ( ((list key _ _ _ _ _) part) )
	     (funcall interpret-fn key)))
	 (when (or (= code display:*right-button*)
		   (= code display:*middle-button*))
	   (display:destroy-window))))
     (display:enter (part)
       (display:set-message-string
	(if part
	    (dlet ( ((list _ message _ _ _ _) part) )
	      (format nil "L:~A; R:Delete this window, M:Both." message))
	    "M,R:Delete this window."))))))

;;; Text is what to display on the screen.
;;; fontkey is a keyword saying what font to display it in.
;;; message is what to put in the message area when the mouse moves over it,
;;; key is what to tell the interpret function when the part is selected.
;;; pointable indicates whether the glyph should be highlightable.
;;; plaintext is the text to use for the glyph if we are converting
;;;    the glyphs to ordinary text instead of displaying them.  
;;;
;;; Bug: the "ordinary text" version will necessarily have the same
;;; number of lines as the glyph version, since the newlines aren't in
;;; the glyphs.  Oh well.
(defsconstr "
    DATATYPE Glyph of
       text : String ,
       fontkey : Symbol ,
       message : String ,
       key : T ,
       pointable : Boolean ,
       plaintext : String
")

;;; Probably ought to rethink this so that displaying glyphs from a
;;; string and displaying glyphs from a parsed file aren't really
;;; different, and so we can display from the file without consing
;;; together a copy of it in memory.
(export 'hypertext-dt)
(export 'font-table)
(def-combined-class hypertext-dt
  display:default-display-term
  (anonymous-class hypertext-dt (font-table glyphs maxheight maxwidth)
    :initable-instance-vars (font-table glyphs)
    :local-funs
    ((map-glyphs (fn)
       ;; For each glyph, pass fn the glyph, its x and y position, and
       ;; its font.
       (dlet* ((y 0))
	 (dotimes (i (length glyphs))
	   (dlet* ((thisline (aref glyphs i))
		   (x 0)
		   (maxheight 0))
	     (dotimes (i (length thisline))
	       (dlet* ( ((glyph _ fontkey _ _ _ _) (elt thisline i))
			((cons _ fontres) (assoc fontkey font-table))
			(_ (unless fontres
			     (error
			      "Font keyword ~s is not found in font table ~s.~%"
			      fontkey font-table)))
			((values _ ascent descent)
			 (display:font-size fontres)) )
		 (setq maxheight (max (+ ascent descent) maxheight))))
	     (dotimes (i (length thisline))
	       (dlet* ( ((:as glyph (glyph text fontkey _ _ _ _))
			 (elt thisline i))
			((cons _ fontres) (assoc fontkey font-table))
			((values width _ descent)
			 (display:font-size fontres)) )
		 (funcall fn glyph x (+ y maxheight (- descent)) fontres)
		 (incf x (* width (length text)))))
	     (incf y maxheight))))))
    :beforeinit
    (progn
      (setq font-table
	    (mapcar (fdlambda ( (cons symbol fontname) ) 
		      (dlet* ( (font (display:open-font fontname)) )
			(cons symbol font)))
		    font-table))
      ;;Now calculate the maximum width and height.
      (setq maxwidth 0 maxheight 0)
      (map-glyphs
       (fdlambda ((glyph string _ _ _ _ _) x y font)
	 (dlet ( ((values width _ descent) (display:font-size font)) )
	   (setq maxwidth (max maxwidth (+ x (* width (length string)))))
	   (setq maxheight (max maxheight (+ y descent)))))))
    :client-methods
    ((display:unparse-for-win (window leftx topy &key width height)
       (declare (ignore width height))
       (map-glyphs
	(fdlambda ( (glyph text _ _ _ _ _) x y font )
	  (display:with-gcontext (display:*write-gcontext* :font font)
	    (display:draw-string window display:*write-gcontext*
				 (+ x leftx) (+ y topy)
				 text)))))
     (display:natural-shape (&key width height)
       (declare (ignore width height))
       (values maxwidth maxheight))
     ;; This is less efficient than it could be.  If it is too slow,
     ;; make it cache a table saying the y coordinate of each line.
     (display:point (x y leftx topy &key width height)
       (declare (ignore width height))
       (decf x leftx)
       (decf y topy)
       (block pointing
	 (map-glyphs
	  (fdlambda ( (glyph text _ message key pointable _) glx gly font )
	    (when pointable
	      (dlet* ( (xd (- x glx))
		       (yd (- y gly))
		       ((values width ascent descent)
			(display:font-size font)) )
		(when (and (>= xd 0) (>= yd (- ascent))
			   (< xd (* width (length text))) (< yd descent))
		  (return-from pointing
		    (list key message (+ glx leftx) (- gly ascent)
			  (* width (length text))
			  (+ ascent descent))))))))
	 nil))
     ;; How is :box to draw a hollow box, anything else to draw a
     ;; filled box.
     (display:highlight-graphics
       (window leftx topy part &key width height how onp)
       (declare (ignore width height onp))
       (declare (ignore leftx topy))
       (when part
	 (dlet ( ((list _ _ x1 y1 x2 y2) part) )
	   (display:draw-rectangle window display:*xor-gcontext* x1 y1 x2 y2
				   (not (eq how :box)))))))))

(defparameter *special-glyph-start* #\{
  "The character that starts special glyphs.")

(defparameter *special-glyph-end* #\}
  "The character that ends special glyphs.")

(defparameter *glyph-arg-start* #\|
  "The character that starts an argument in a glyph.")

(defparameter *glyph-escape* #\\
  "Escape character.")

;;; Need to rethink this so that it works on sequential files without
;;; having to cons a copy of the file in memory.
;;; Instead of vector-push-extending to generate our results, we
;;; should be vector-push-extending a buffer and copying the buffer to
;;; our result (of course, if we were doing the right thing with
;;; sequential files, this would not be an issue.)
(defun parse-glyphs (string &key (fill-glyph
				  #'(lambda (&rest ignore)
				      (declare (ignore ignore)))))
  (let ((string-pos 0))
    (labels
	((peek-char () (elt string string-pos))
	 (next-char () (incf string-pos))
	 (seeing (ch) (char-equal (peek-char) ch))
	 (vecpush (vec) (vector-push-extend (peek-char) vec))
	 (endp () (>= string-pos (length string)))
	 (eolp () (seeing #\newline))
	 (make-pushable-string () (make-array '(0) :adjustable t
				     :fill-pointer t
				     :element-type 'character))
	 (pushable-to-string (str) (concatenate 'string str))
	 (debug-print (exp str)
	   (format t "~a is ~s.~%" str exp)
	   exp)
	 (push-check-escape (result)
	   (if (seeing *glyph-escape*)
	       (progn
		 (next-char)
		 (cond
		  ((or (endp) (eolp)))
		  ((seeing #\') (vector-push-extend #\" result))
		  (t (vecpush result))))
	       (vecpush result))
	   (next-char))
	 (parse-warning (format-str &rest format-args)
	   ;; Someday make this print out a pointer so that people
	   ;; won't have to count hundreds of characters.  Maybe take
	   ;; code from SB's lexer.
	   (warn "In parse-glyphs, parsing~%~s~% at position ~s:~%~a~%"
		 string string-pos
		 (apply #'format nil format-str format-args)))
	 (parse-unspecial-text ()
	   (let ((result (make-pushable-string)))
	     (loop
	      (when (or (endp) (eolp) (seeing *special-glyph-start*))
		(return (pushable-to-string result)))
	      (when (seeing *special-glyph-end*)
		(parse-warning "Extra ~a here." *special-glyph-end*))
	      (push-check-escape result))))
	 (parse-special-text (warn-on-newlines keywordp)
	   (let ((result (make-pushable-string)))
	     (loop
	      (when (or (endp) (seeing *special-glyph-end*)
			(seeing *glyph-arg-start*)
			(and keywordp (seeing #\ )))
		(return (pushable-to-string result)))
	      (when (seeing *special-glyph-start*)
		(parse-warning "Extra ~a here." *special-glyph-end*))
	      (if (and (eolp) warn-on-newlines)
		  (progn
		    (parse-warning "Newline in special glyph ignored.")
		    (next-char))
		  (push-check-escape result)))))
	 (parse-special-args ()
	   (let ((result '()))
	     (loop
	      (when (or (endp) (seeing *special-glyph-end*))
		(return (nreverse result)))
	      (assert (seeing *glyph-arg-start*))
	      (next-char)
	      (push (intern (string-upcase (parse-special-text t t)) "KEYWORD")
		    result)
	      (if (or (seeing *glyph-arg-start*) (seeing *special-glyph-end*))
		  (push "" result)
		  (progn
		    (assert (seeing #\ ) () "Expected a space, saw a ~s."
			    (peek-char))
		    (next-char)
		    (push (parse-special-text nil nil) result))))))
	 (make-default-glyph ()
	   (glyph "" :normal "" nil nil ""))
	 (parse-glyph ()
	   (let ((result (make-default-glyph)))
	     (if (seeing *special-glyph-start*)
		 (progn
		   (next-char)
		   (apply fill-glyph result
			  (parse-special-text t nil) t (parse-special-args))
		   (next-char))
		 (funcall fill-glyph result (parse-unspecial-text)
			  nil))
	     result))
	 (parse-line ()
	   (let ((result (make-array '(0) :adjustable t :fill-pointer t)))
	     (loop
	      (when (or (endp) (eolp)) (return result))
	      (vector-push-extend (parse-glyph) result))))
	 (parse-string ()
	   (let ((result (make-array '(0) :adjustable t :fill-pointer t)))
	     (loop
	      (when (endp) (return result))
	      (vector-push-extend (parse-line) result)
	      (assert (or (endp) (eolp)))
	      (next-char)))))
      (parse-string))))

(export 'de-parse-glyphs)
(defun de-parse-glyphs (glyphs)
  (with-output-to-string (s)
    (dotimes (i (length glyphs))
      (let ((line (elt glyphs i)))
	(dotimes (j (length line))
	  (format s "~a" (glyph-plaintext (elt line j))))
	(unless (= i (- (length glyphs) 1))
	  (format s "~%"))))))

(export 'add-mellowcard-escapes)
(defun add-mellowcard-escapes (string)
  (let ((result (make-array '(0) :adjustable t
				     :fill-pointer t
				     :element-type 'character)))
    (labels ((pushres (ch) (vector-push-extend ch result)))
      (dotimes (i (length string))
	(let ((thisch (elt string i)))
	  (case thisch
	    ((#\\ #\{ #\}) (pushres #\\) (pushres thisch))
	    (t (pushres thisch))))))
    (concatenate 'string result)))

#| Test case.
(defun play-with-glyphs (glyphs)
  (display:letrec-hierarchy
      ((mouse-win (display:vertical-stacking-message-window :name "Mouse Help Window"))
       (hyper-win (hypertext-win
		   :title "Test case."
		   :message-window mouse-win
		   :display-term
		   (make-instance hypertext-dt
				  :glyphs glyphs
				  :font-table
				  '((:normal . #+x11 "vtsingle" #-x11 "4x6")
				    (:bold . #+x11 "8x13bold" #-x11 "oldengssx")
				    (:huge . #+x11 "vr-40" #-x11 "noni44")))
		   :interpret-fn
		   #'(lambda (stuff)
		       (case stuff
			 (:delete
			  (display:destroy-window hyper-win)
			  (display:destroy-window mouse-win))
			 (t (eval stuff)))))))
    (display:wait-for-mouse-events)))

(progn
  (labels ((myglyph (text fontkey message key pointable)
	     (glyph text fontkey message key pointable text)))
    (defparameter *test-glyphs*
      (vector
       (vector (myglyph "Pick this " :normal "" nil nil)
	       (myglyph "string" :bold "Say hi." '(format t "Hi.~%") t)
	       (myglyph " for something " :normal "" nil nil)
	       (myglyph "interesting" :bold "Say there." '(format t "There.~%") t)
	       (myglyph " to happen." :normal "" nil nil))
       (vector (myglyph "Pick this " :normal "" nil nil)
	       (myglyph "string" :bold "Say hi." '(format t "Hi.~%") t)
	       (myglyph " for something similar but nevertheless also "
			:normal "" nil nil)
	       (myglyph "interesting" :huge "Say there." '(format t "There.~%") t)
	       (myglyph " to happen." :normal "" nil nil))
       (vector (myglyph "Pick this when you are done: " :normal "" nil nil)
	       (myglyph "Done." :bold "Delete all windows." :delete t)))))
  (play-with-glyphs *test-glyphs*))

(progn
  (let ((parsed (parse-glyphs
"Pick this {string|key hi} for something {interesting|key there} to happen.
Pick this {string|key hi} for something similar but nevertheless also \\
{interesting|font huge|key there} to happen.
{Pick this when you are done.}
Slash: \\\\
Say slash: {\\\\|key \\\\}
Brace: \\{
Say right brace: {\\}|key \\}}
Say bar: {\\||key \\|}
Empty key: {hi|key}
{This should get a parse warning
|key Parse warning.}
Another parse warning:}"
	:fill-glyph
	#'(lambda (glyph text special-p
			 &key (key "" key-sp) (font "bold" font-sp))
	    (setf (glyph-text glyph) text)
	    (setf (glyph-plaintext glyph) text)
	    (setf (glyph-fontkey glyph)
		   (if special-p
		       (intern (string-upcase font) "KEYWORD")
		       :normal))
	    (setf (glyph-message glyph)
		  (cond
		   ((not special-p) "")
		   (key-sp (format nil "Say ~a." key))
		   (t (format nil "Delete this window."))))
	    (setf (glyph-key glyph)
		  (if key-sp
		      `(format t "~s.~%" ',key)
		      :delete))
	    (setf (glyph-pointable glyph) special-p)))))
    (play-with-glyphs parsed))) 
|#
