module mellowdb.

import mellowscreen.

type mellowcard_top hypertext_subject.
defhypertext mellowcard_top "MellowCard"
 "MellowCard is ``the hypertext system that doesn't try as hard''.
More specifically, it allows you to easily imbed mouse-sensitive
strings in displayed text and control what happens when the mouse is
clicked on these special strings.  You specify the mouse-sensitive
strings by putting special punctuation in your text.  
 
A MellowCard string is a string with these special punctuation marks
in it.  This punctuation is described in \
{MellowCard Strings|key mellowcard_strings}.  
 
A MellowCard screen is a piece of information residing in memory that
may eventually become visible on your display device.
 
A MellowCard window is the entity that displays a MellowCard screen on
your display device.
 
For a description of the various kinds of predefined MellowCard
screens, see {Predefined MellowCard Screens|key predefined_mellowcard_screens}.
 
For a description of how you can make your own MellowCard screens from
scratch, see the documentation for {defscreen|action win_describe
defscreen}.
 
For self-congratulatory prose from the author, see \
{Programming Notes|key mellowcard_programming_notes}.".
 
type mellowcard_programming_notes hypertext_subject.
defhypertext mellowcard_programming_notes "Programming Notes"
 "The lisp and lprolog code for MellowCard (with the exception of the
self-documentation) were written in rapid-prototyping mode in almost
one weekend (Friday night through Monday night) on February 24-28
1989.  The following improvements are contemplated in the indefinite
future:
 
   Making the code more efficient
   Displaying from text and DVI files 
   Automatic word wrapping
   Dealing with variable-width fonts
   Giving the windows a better default position (not always in the
      upper left corner of the screen).
   Improving the existing describe command to remove the MellowCard
      punctuation from the strings.
   Make help_topics generate multiple lines of predicate names if
      there are an unreasonable number of them. 
   Package up the persistent mouse-help line and make it available
      from lisp.
   Eliminate the code in mellowlam that is redundant with code in the
      SB's lexer.
 
Send bug reports to Tim Freeman at tsf@cs.cmu.edu.  No guarantees
here; your bug report may very well wind up being added to the above
list of indefinitely postponed improvements.
 
The system is fairly simple now.  Hacking around with it to make
something better out of it is encouraged.".

type mellowcard_strings hypertext_subject.
defhypertext mellowcard_strings "MellowCard Strings"
 "MellowCard strings consist of ordinary text mixed with special glyphs
enclosed in braces.  The general form of a special glyph is

   \{text|key1 value1|key2 value2...\}

For more details about keywords, see \
{MellowCard Keywords|key mellowcard_keywords}.  

Sometimes you will want to include characters in a MellowCard string
which are special in some sense.  This includes \' (because lambda
prolog will not normally terminate the string when it gets to a quote)
and \{ (because it starts glyphs).  For a description of how to do
this, see {MellowCard Escapes|key mellowcard_escapes}.".

type mellowcard_escapes hypertext_subject.
defhypertext mellowcard_escapes "MellowCard Escapes"
 "\\ is the escape character in MellowCard strings.  The following
 escape sequences are useful:
   Sequence                 Effect
     \\'                       \'
     \\<newline>               <empty>
     \\<anything else>         <anything else>".

type mellowcard_keywords hypertext_subject.
defhypertext mellowcard_keywords "MellowCard Keywords"
 "This screen describes the available mellowcard keywords.  The
examples are live; you can click on them with the mouse and
reasonable things will happen.  This is a hypertext screen, so the
\'default action\' in the examples is \
{show_hypertext|action win_describe show_hypertext}.

The following keywords are available:

Keyword       Example             Effect  Description   
<none>        \{foo\}                {foo}    Parse the text of the glyph
                                          into lambda prolog, and pass
                                          the result to the default action.  
|key          \{foo|key bar\}        {foo|key bar}    Parse the given value into
                                          lambda prolog and pass the
                                          result to the default action.  
|action       \{foo|action write 3\} {foo|action write 3}    Parse the given value and solve
                                          it.
|message      \{foo|message Hi\}     {foo|message Hi}    Put the message in the mouse help
                                          window while the mouse is on this
                                          glyph.
|lisp-action  \{foo|lisp-action     {foo|lisp-action (print 3)}    Evaluate the given lisp-action.
                        (print 3)\}
|plaintext    \{foo|plaintext bar\}  {foo|plaintext bar}    When {ordinarify_mellowcard_text|action win_describe ordinarify_mellowcard_text}
                                          is called, the text here
                                          should be bar instead of
                                          foo.
|font         \{foo|font normal\}    {foo|font normal}    Display in the given font.
                                          Currently available fonts
                                          include \'bold\' and \'normal\'.
                                          Adding new ones is a trivial
                                          lisp hack.
|nohigh       \{foo|nohigh\}         {foo|nohigh}    Make the glyph unselectable.
It is easy to add new keywords by modifying the Lisp code.".

type foo hypertext_subject.
defhypertext foo "Foo"
 "{Click here to return to MellowCard Keywords.|key mellowcard_keywords}".

type bar hypertext_subject.
defhypertext bar "Bar"
 "{Click here to return to MellowCard Keywords.|key mellowcard_keywords}".

type predefined_mellowcard_screens hypertext_subject.
defhypertext predefined_mellowcard_screens "Predefined MellowCard Screens"
 "There are two kinds of predefined {MellowCard|key mellowcard_top} screens:
hypertext screens and description screens.

Hypertext screens simply have a name, a title, and text, and they are
defined by {defhypertext|action win_describe defhypertext}.  The default \
action for special
glyphs in a hypertext screen is to call \
{show_hypertext|action win_describe show_hypertext} 
on them.

Description screens describe lambda prolog items.  The title and some
of the text of the screen are generated automatically from the item
being described.  The programmer may use {doc|action win_describe doc} \
to specify the ordinary
prose part of a description screen.  The default action for special
glyphs in a description screen is to call \
{win_describe|action win_describe win_describe} on them.".

type mellowtop o.

mellowtop :-
   "mellowlow" ==> ("mellowscreen" ==> ("mellowdesc" ==> ("mellowdb" ==> top))).
