module mellowdesc.

import help.
import mellowlow.
import mellowscreen.

type	writing_to_mellowcard_win Title -> (Glyph -> o) ->
                                  string -> string -> o -> o.
defined	writing_to_mellowcard_win mellowcard.
mellowdoc	writing_to_mellowcard_win
 "``writing_to_mellowcard_win Title Default_act Before_string After_string Goal''
solves Goal and puts all of the output generated by Goal into a
{MellowCard|action show_hypertext mellowcard_top} window.  Default gets solved \
each time a special glyph from
the mellowcard window is invoked that has no |action keyword.

Before_string and after_string are appended around the text of the
glyph to produce the default message.  See also {show_mellowcard_text}.".
writing_to_mellowcard_win Title Default_act Before_string After_string Goal :-
   writing_to_string Goal Text,
     show_mellowcard_text Title
			  Default_act
			  Before_string
			  After_string
			  Text.

type	removing_mellowcard_commands o -> o.
defined	removing_mellowcard_commands mellowcard.
mellowdoc	removing_mellowcard_commands
 "``removing_mellowcard_commands Goal'' solves Goal, converts all
{mellowcard|action show_hypertext mellowcard_top} commands in the output \
generated by Goal to ordinary text,
then writes the result to the standard output.".
removing_mellowcard_commands Goal :-
   writing_to_string Goal X,
   ordinarify_mellowcard_text X Y,
   writesans Y.

%%% Help predicates.
type	win_help	o.
defined win_help	elp_help.
mellowdoc	win_help
 "Behaves like the {help} command, except puts the information in a
{mellowcard|action show_hypertext mellowcard_top} window.".

type	win_describe	A -> o.
defined	win_describe 	elp_help.
mellowdoc	win_describe	
 "``win_describe Object'' puts a description of Object in a \
 {MellowCard|action show_hypertext mellowcard_top}
window.  This uses the same database as {describe}.  The default
action for special glyphs in the window is to recursively call
win_describe on them.".

type    win_list_help_topics o.
type    win_describe_topic elp_topic -> o.
defined win_describe_topic elp_help.
mellowdoc	win_describe_topic
 "``win_describe_topic Topic'' lists all describable specials, defined
predicates, and constants for the given topic in a \
{MellowCard|action show_hypertext mellowcard_top}
window.".

win_help :-
 writing_to_mellowcard_win "Help" win_describe_topic "List items in topic " ""
    ( writesans 
"This is eLP Version 0.  
Click on one of the following help topics 
for a list of eLP items for that topic.", nl,
      win_list_help_topics ).

win_list_help_topics :- 
   topic T String, 
   nl,
   writesans "{",
   writesans String,
   writesans "|key ",
   write T,
   writesans "}",
   fail.
win_list_help_topics.

type    win_help_category   (A -> elp_topic -> o) -> string -> elp_topic -> o.

win_describe_topic T :-
   topic T String,
   writing_to_string (writesans "Help topics for ", writesans String) Title,
   writing_to_mellowcard_win Title win_describe "Describe " ""
      (writesans "special predicates: ",
	       win_help_category special "special predicates" T,
               writesans "; ", nl,
       writesans "defined predicates: ",
               win_help_category defined "defined predicates" T,
               writesans "; ", nl,
       writesans "declared constants: ",
               win_help_category const "declared constants" T,
               writesans ".").

win_help_category Cat Cat_string T :-
  not (sigma Sp \ (Cat Sp T)), !,
  writesans " [No ", writesans Cat_string, writesans "]".
win_help_category Cat Cat_string T :- 
  Cat X T, writesans " {", writeered X, writesans "}", fail.
win_help_category Cat Cat_string T.

type	description_screen A -> mellowcard_screen.
% Hide this from the user.
%const	description_screen mellowcard.
%mellowdoc	description_screen
% "``description_screen Subject'' constructs a \
%{mellowcard|action show_hypertext mellowcard_top} screen that 
%displays the result of describing the given Subject.  See also
%{describe}, {win_describe}, and {show_mellowcard_screen}.".

write_doc_string X :-
   mellowdoc X String, !,
   ordinarify_mellowcard_text String Result,
   writesans Result.

type write_mellowdoc_string A -> o.
write_mellowdoc_string X :-
   mellowdoc X String, !, writesans String.

write_mellowdoc_string X :-
   doc X String, !, mellowfy_ordinary_text String Result, writesans Result.

write_mellowdoc_string X :- writesans "No documentation available.".

type low_win_describe A -> o.
type low_win_describe0 A -> o.
type win_plain_describe A -> o.
% The difference between low_win_describe and describe is that
% low_win_describe uses write_mellowdoc_string where describe uses
% write_doc_string.
low_win_describe X :- switching (warning type_unify) off (low_win_describe0 X).
low_win_describe0 X :- special X Topic,
	      nl, writeered X, writesans " is a special predicate of topic ",
	      write_topic Topic, fail.
low_win_describe0 X :- defined X Topic,
	      nl, writeered X, writesans " is a defined predicate of topic ",
	      write_topic Topic, fail.
low_win_describe0 X :- const X Topic,
	      nl, writeered X, writesans " is a declared constant of topic ",
	      write_topic Topic, fail.
low_win_describe0 X :- win_plain_describe X.

win_plain_describe X :- 
  nl, writeered X, writesans " : ", writetype X,
  nl, write_mellowdoc_string X.

defscreen (description_screen Subject) Title win_describe
          "Describe " "" Text :-
   writing_to_string (writesans "Description of ", writeered Subject) Title,
   writing_to_string (low_win_describe Subject) Text.

win_describe Subject :-
   show_mellowcard_screen (description_screen Subject).

