module mellowlow.

% First the interface to the hypertext stuff.
type	mellowcard	elp_topic.
topic	mellowcard	"MellowCard".

type	mellowdoc	A -> string -> o.
mellowdoc mellowdoc
 "``mellowdoc C String'' succeeds if C is a constant that has been
given the {MellowCard|action showscreen mellowcard} \
documentation string String.  This is different
from {doc} because strings specified with mellowdoc are assumed to
have MellowCard commands in them, whereas strings specified with doc
are assumed to be plain text.".

type	writing_to_string	o -> string -> o.
special	writing_to_string	output.
mellowdoc writing_to_string 
 "``writing_to_string Goal String'' executes Goal and collects all of
the output generated (by using {writesans}, {write}, {nl},
{writeered}, or other output primitives) into a string which it then
unifies with String.  For example,

   writing_to_string (write 3, writesans \' is a number\') X

unifies X with \'3 is a number\'.".

% Shows text on a mellowcard window.
type	show_mellowcard_text	Title -> (Glyph -> o) -> 
                                string -> string ->
				string -> o.
defined show_mellowcard_text	mellowcard.
mellowdoc     show_mellowcard_text
 "``show_mellowcard_text Title Default_act Before_message After_message Text''
creates a {MellowCard|action showscreen mellowcard} window.
   Title is the title of the window.
   Default_act is an lprolog predicate to call when a glyph in the window
      is clicked upon that has no |action or |lisp-action keyword,
   Before_message and After_message are appended to the text of the
      glyph to form a message to give the user a hint about what will
      happen if the glyph is clicked upon.
   Text is the text to display in the window.
See also {writing_to_mellowcard_win}.

I (TSF) wish that instead of Before_message and After_message there
were an lprolog predicate that maps the glyph text to the default message
string.  As of 28 Feb 1989 I haven't done this because it is difficult
to make lprolog specials that take arguments of type A -> B -> o.".

% This indirection here is to work around a "feature" that requires
% arguments of type A -> o to specials to be eta-expanded. 
type show_mellowcard_text_low Title -> (Glyph -> o) -> 
                                string -> string ->
				string -> o.
show_mellowcard_text Title Default_act Before After Text :-
   show_mellowcard_text_low Title
      (Thing \ (Default_act Thing)) Before After Text.

% Converts mellowcard text to ordinary text.
type	ordinarify_mellowcard_text string -> string -> o.
special ordinarify_mellowcard_text mellowcard.
mellowdoc	ordinarify_mellowcard_text
 "``ordinarify_mellowcard_text Before After'' removes special
{MellowCard|action show_hypertext mellowcard_top} punctuation \
from Before, and unifies the result with
After.  The \'ordinary\' version of a glyph is the text that
the glyph would display on the screen unless the glyph has a
|plaintext keyword.  If the glyph has a \|plaintext keyword, the value
of this keyword is used instead of the text of the glyph.  For
example,
 
   ordinarify_mellowcard_text \'abc \{def\} \{ghi\|plaintext klm\}\' X.
 
unifies X with the string \'abc def klm\'.
 
For more about MellowCard keywords (like |plaintext), see \
{MellowCard Keywords|action show_hypertext mellowcard_keywords}.
See also {mellowfy_ordinary_text}.".

type mellowfy_ordinary_text string -> string -> o.
special mellowfy_ordinary_text mellowcard.
mellowdoc mellowfy_ordinary_text
 "``mellowfy_ordinary_text Before After'' copies Before to After
quoting any {MellowCard|action show_hypertext mellowcard_top} \
escape characters.  Thus, if After is
displayed on a MellowCard window, then it has the same appearance
that Before would have if it were written to the standard output.
For example, 

   mellowfy_ordinary_text \'\{\\ab\' X.
 
unifies X with the string \'\\\{\\\\ab\'.

For a description of MellowCard escape sequences, see \
{MellowCard Escapes|action show_hypertext  mellowcard_escapes}.
See also {ordinarify_mellowcard_text}.".
