module mellowscreen.

import mellowlow.

kind	mellowcard_screen type.
type	defscreen mellowcard_screen -> string -> (Glyph -> o)
-> string -> string -> string -> o.
defined defscreen mellowcard.
mellowdoc defscreen
 "``defscreen Name Title Default_act Before_message After_message Text''
defines a {MellowCard|action show_hypertext mellowcard_top} \
screen.  Name is the name of the screen, to be
used in a later call to {show_mellowcard_screen}.  Default_act,
Before_message, After_message, and Text have the same purpose as the
corresponding arguments to {show_mellowcard_text}.".

type show_mellowcard_screen mellowcard_screen -> o.
% Since the only instances we have right now of mellowcard screens are
% hypertext screens and documentation screens, I (TSF) think that the idea of
% mellowcard_screen is not useful enough to be worth while showing the
% documenation to the user.
%
defined show_mellowcard_screen mellowcard.
mellowdoc show_mellowcard_screen
 "``show_mellowcard_screen Screen'' displays a \
{MellowCard|action show_hypertext mellowcard_top} screen defined with 
{defscreen}.  There are general clauses for defscreen that cause objects
constructed with {defhypertext} and {doc} to both be MellowCard screens.".

show_mellowcard_screen Screen :-
   defscreen Screen Title Default_act Before_message After_message Text,
   show_mellowcard_text Title Default_act Before_message After_message Text.

kind	  hypertext_subject type.
type	  hypertext_screen  hypertext_subject -> mellowcard_screen.

type	  defhypertext      hypertext_subject -> string -> string -> o.
defined	  defhypertext	    mellowcard.
mellowdoc	  defhypertext
 "``defhypertext Subject Title Text'' defines a hypertext screen named
Screen with the given Title and Text.  The default action for special
glyphs in this window is to call {show_hypertext} on them.  See 
{Special Mellowcard Windows|action show_hypertext special_mellowcard_windows} \
for a comparison of hypertext, documentation, and general mellowcard
windows.  For an example, see \
{Example Hypertext Window|action show_hypertext hypertext_example}.
For a list of all defined hypertext screens, use {show_all_hypertext} or
click {here|action show_all_hypertext}.".

type	  show_hypertext    hypertext_subject -> o.
defined	  show_hypertext    mellowcard.
mellowdoc	  show_hypertext    
 "``show_hypertext Subject'' displays the hypertext screen named
Subject.  See also {defhypertext}.  For an example, see 
{Example Hypertext Window|action show_hypertext hypertext_example}.
For a list of all defined hypertext screens, use {show_all_hypertext} or
click {here|action show_all_hypertext}.".

defscreen (hypertext_screen Subject) Title show_hypertext "Display subject " ""
     Text :-
   defhypertext Subject Title Text.

show_hypertext Subject :- show_mellowcard_screen (hypertext_screen Subject).

type all_hypertext mellowcard_screen.

type all_hypertext_iterate o.
defscreen all_hypertext "All Defined Hypertext Screens" show_hypertext 
   "Display subject " "" Text :-
 writing_to_string
    (writesans "Subject", tab 30, writesans "Title",nl,nl,
     all_hypertext_iterate) Text.

all_hypertext_iterate :-
   defhypertext Name Title Text,
   writesans "{",write Name, writesans "}",tab 32, writesans Title, nl,
   fail.

all_hypertext_iterate.

type show_all_hypertext o.
defined show_all_hypertext mellowcard.
mellowdoc show_all_hypertext 
 "``show_all_hypertext'' displays a table of all defined hypertext
screens.  To run it now, click {here|action show_all_hypertext}".
show_all_hypertext :- show_mellowcard_screen all_hypertext.

type	hypertext_example hypertext_subject. 
defhypertext hypertext_example "Hypertext Example"
 "This is a sample hypertext screen.  It was defined by the commands

   type hypertext_example hypertext_subject.
   defhypertext hypertext_example \'Hypertext Example\' 
      \'This is a sample hypertext screen.  It ...\'.

Hypertext screens can reference other hypertext screens (or even
themselves) by simply having a special glyph; for instance, the text

   \{hypertext_example\}

generates \'{hypertext_example}\', and the text

   \{Hypertext Example|key hypertext_example\}

generates \'{Hypertext Example|key hypertext_example}\'.

All hypertext windows are also mellowcard windows.  

For a general discussion of MellowCard, see \
{MellowCard|key mellowcard_top}.  

For a description of the differences between general mellowcard screens,
hypertext screens, and description screens, see \
{Predefined MellowCard Screens|key predefined_mellowcard_screens}.

For a more detailed description of glyphs, see \
{MellowCard Strings|key mellowcard_strings}.".
